@@ -9,7 +9,7 @@ from redis.connection import DefaultParser
 
 from . import _msgs as msgs
 from ._command_args_parsing import extract_args
-from ._commands import Int, Float, SUPPORTED_COMMANDS, COMMANDS_WITH_SUB, Item, Signature
+from ._commands import Int, Float, SUPPORTED_COMMANDS, COMMANDS_WITH_SUB, Item, Signature, CommandItem
 from ._helpers import (
     SimpleError,
     valid_response_type,
@@ -18,8 +18,7 @@ from ._helpers import (
     casematch,
     compile_pattern,
     QUEUED,
-    decode_command_bytes,
-)
+    decode_command_bytes, )
 from ._stream import XStream
 from ._zset import ZSet
 
@@ -66,7 +65,7 @@ class BaseFakeSocket:
     }
     _connection_error_class = redis.ConnectionError
 
-    def __init__(self, server, db, *args, **kwargs):
+    def __init__(self, server: "FakeServer", db: int, *args: Any, **kwargs: Any):  # noqa: F821
         super(BaseFakeSocket, self).__init__(*args, **kwargs)
         from fakeredis import FakeServer
         self._server: FakeServer = server
@@ -84,6 +83,7 @@ class BaseFakeSocket:
         self._transaction: Optional[List[Any]]
         self._in_transaction: bool
         self._pubsub: int
+        self._transaction_failed: bool
 
     def put_response(self, msg: Any) -> None:
         """Put a response message into the queue of responses.
@@ -126,7 +126,7 @@ class BaseFakeSocket:
             subs.discard(self)
         self._clear_watches()
 
-    def close(self):
+    def close(self) -> None:
         # Mark ourselves for cleanup. This might be called from
         # redis.Connection.__del__, which the garbage collection could call
         # at any time, and hence we can't safely take the server lock.
@@ -199,7 +199,7 @@ class BaseFakeSocket:
     def _decode_error(self, error):
         return DefaultParser(socket_read_size=65536).parse_error(error.value)  # type: ignore
 
-    def _decode_result(self, result):
+    def _decode_result(self, result: Any) -> Any:
         """Convert SimpleString and SimpleError, recursively"""
         if isinstance(result, list):
             return [self._decode_result(r) for r in result]
@@ -210,7 +210,7 @@ class BaseFakeSocket:
         else:
             return result
 
-    def _blocking(self, timeout: Optional[Union[float, int]], func: Callable):
+    def _blocking(self, timeout: Optional[Union[float, int]], func: Callable[[bool], Any]):
         """Run a function until it succeeds or timeout is reached.
 
         The timeout is in seconds, and 0 means infinite. The function
@@ -359,7 +359,7 @@ class BaseFakeSocket:
             result_cursor = 0
         return [str(bin_reverse(result_cursor, bits_len)).encode(), result_data]
 
-    def _ttl(self, key, scale) -> int:
+    def _ttl(self, key: CommandItem, scale: int) -> int:
         if not key:
             return -2
         elif key.expireat is None:
