@@ -1,10 +1,12 @@
 import functools
 import hashlib
+import importlib
 import itertools
 import logging
+import os
 from typing import Callable, AnyStr, Set, Any, Tuple, List, Dict, Optional
 
-from lupa import LuaRuntime
+import lupa
 
 from fakeredis import _msgs as msgs
 from fakeredis._commands import command, Int, Signature
@@ -16,6 +18,16 @@ from fakeredis._helpers import (
     decode_command_bytes,
 )
 
+__LUA_RUNTIMES_MAP = {
+    "5.1": "lupa.lua51",
+    "5.2": "lupa.lua52",
+    "5.3": "lupa.lua53",
+    "5.4": "lupa.lua54",
+}
+LUA_VERSION = os.getenv("FAKEREDIS_LUA_VERSION", "5.1")
+
+with lupa.allow_lua_module_loading():
+    LUA_MODULE = importlib.import_module(__LUA_RUNTIMES_MAP[LUA_VERSION])
 LOGGER = logging.getLogger("fakeredis")
 REDIS_LOG_LEVELS = {
     b"LOG_DEBUG": 0,
@@ -39,7 +51,7 @@ def _ensure_str(s: AnyStr, encoding: str, replaceerr: str) -> str:
     return res
 
 
-def _check_for_lua_globals(lua_runtime: LuaRuntime, expected_globals: Set[Any]) -> None:
+def _check_for_lua_globals(lua_runtime: LUA_MODULE.LuaRuntime, expected_globals: Set[Any]) -> None:
     unexpected_globals = set(lua_runtime.globals().keys()) - expected_globals
     if len(unexpected_globals) > 0:
         unexpected = [
@@ -48,7 +60,7 @@ def _check_for_lua_globals(lua_runtime: LuaRuntime, expected_globals: Set[Any])
         raise SimpleError(msgs.GLOBAL_VARIABLE_MSG.format(", ".join(unexpected)))
 
 
-def _lua_redis_log(lua_runtime: LuaRuntime, expected_globals: Set[Any], lvl: int, *args: Any) -> None:
+def _lua_redis_log(lua_runtime: LUA_MODULE.LuaRuntime, expected_globals: Set[Any], lvl: int, *args: Any) -> None:
     _check_for_lua_globals(lua_runtime, expected_globals)
     if len(args) < 1:
         raise SimpleError(msgs.REQUIRES_MORE_ARGS_MSG.format("redis.log()", "two"))
@@ -69,12 +81,22 @@ class ScriptingCommandsMixin:
     _run_command: Callable[[Callable[..., Any], Signature, List[Any], bool], Any]
 
     def __init__(self, *args: Any, **kwargs: Any):
-        super(ScriptingCommandsMixin, self).__init__(*args, **kwargs)
-        # Maps SHA1 to the script source
-        self.script_cache: Dict[bytes, bytes] = {}
+        self.script_cache: Dict[bytes, bytes] = dict()  # Maps SHA1 to the script source
         self.version: Tuple[int]
+        self.load_lua_modules = set()
+        lua_modules_set: Set[str] = kwargs.pop("lua_modules", None) or set()
+        if len(lua_modules_set) > 0:
+            lua_runtime: LUA_MODULE.LuaRuntime = LUA_MODULE.LuaRuntime(encoding=None, unpack_returned_tuples=True)
+            for module in lua_modules_set:
+                try:
+                    lua_runtime.require(module.encode())
+                    self.load_lua_modules.add(module)
+                except LUA_MODULE.LuaError as ex:
+                    LOGGER.error(f'Failed to load LUA module "{module}", make sure it is installed: {ex}')
 
-    def _convert_redis_arg(self, lua_runtime: LuaRuntime, value: Any) -> bytes:
+        super(ScriptingCommandsMixin, self).__init__(*args, **kwargs)
+
+    def _convert_redis_arg(self, lua_runtime: LUA_MODULE.LuaRuntime, value: Any) -> bytes:
         # Type checks are exact to avoid issues like bool being a subclass of int.
         if type(value) is bytes:
             return value
@@ -89,7 +111,7 @@ class ScriptingCommandsMixin:
             )
             raise SimpleError(msg)
 
-    def _convert_redis_result(self, lua_runtime: LuaRuntime, result: Any) -> Any:
+    def _convert_redis_result(self, lua_runtime: LUA_MODULE.LuaRuntime, result: Any) -> Any:
         if isinstance(result, (bytes, int)):
             return result
         elif isinstance(result, SimpleString):
@@ -111,9 +133,7 @@ class ScriptingCommandsMixin:
             )
 
     def _convert_lua_result(self, result: Any, nested: bool = True) -> Any:
-        from lupa import lua_type
-
-        if lua_type(result) == "table":
+        if LUA_MODULE.lua_type(result) == "table":
             for key in (b"ok", b"err"):
                 if key in result:
                     msg = self._convert_lua_result(result[key])
@@ -141,7 +161,8 @@ class ScriptingCommandsMixin:
             return 1 if result else None
         return result
 
-    def _lua_redis_call(self, lua_runtime: LuaRuntime, expected_globals: Set[Any], op: bytes, *args: Any) -> Any:
+    def _lua_redis_call(
+            self, lua_runtime: LUA_MODULE.LuaRuntime, expected_globals: Set[Any], op: bytes, *args: Any) -> Any:
         # Check if we've set any global variables before making any change.
         _check_for_lua_globals(lua_runtime, expected_globals)
         func, sig = self._name_to_func(decode_command_bytes(op))
@@ -149,7 +170,9 @@ class ScriptingCommandsMixin:
         result = self._run_command(func, sig, new_args, True)
         return self._convert_redis_result(lua_runtime, result)
 
-    def _lua_redis_pcall(self, lua_runtime: LuaRuntime, expected_globals: Set[Any], op: bytes, *args: Any) -> Any:
+    def _lua_redis_pcall(
+            self, lua_runtime: LUA_MODULE.LuaRuntime, expected_globals: Set[Any], op: bytes,
+            *args: Any) -> Any:
         try:
             return self._lua_redis_call(lua_runtime, expected_globals, op, *args)
         except Exception as ex:
@@ -157,30 +180,29 @@ class ScriptingCommandsMixin:
 
     @command((bytes, Int), (bytes,), flags=msgs.FLAG_NO_SCRIPT)
     def eval(self, script: bytes, numkeys: int, *keys_and_args: bytes) -> Any:
-        from lupa import LuaError, LuaRuntime, as_attrgetter
-
         if numkeys > len(keys_and_args):
             raise SimpleError(msgs.TOO_MANY_KEYS_MSG)
         if numkeys < 0:
             raise SimpleError(msgs.NEGATIVE_KEYS_MSG)
         sha1 = hashlib.sha1(script).hexdigest().encode()
         self.script_cache[sha1] = script
-        lua_runtime: LuaRuntime = LuaRuntime(encoding=None, unpack_returned_tuples=True)
-
+        lua_runtime: LUA_MODULE.LuaRuntime = LUA_MODULE.LuaRuntime(encoding=None, unpack_returned_tuples=True)
+        modules_import_str = "\n".join([f"{module} = require('{module}')" for module in self.load_lua_modules])
         set_globals = lua_runtime.eval(
-            """
+            f"""
             function(keys, argv, redis_call, redis_pcall, redis_log, redis_log_levels)
-                redis = {}
+                redis = {{}}
                 redis.call = redis_call
                 redis.pcall = redis_pcall
                 redis.log = redis_log
                 for level, pylevel in python.iterex(redis_log_levels.items()) do
                     redis[level] = pylevel
                 end
-                redis.error_reply = function(msg) return {err=msg} end
-                redis.status_reply = function(msg) return {ok=msg} end
+                redis.error_reply = function(msg) return {{err=msg}} end
+                redis.status_reply = function(msg) return {{ok=msg}} end
                 KEYS = keys
                 ARGV = argv
+                {modules_import_str}
             end
             """
         )
@@ -191,7 +213,7 @@ class ScriptingCommandsMixin:
             functools.partial(self._lua_redis_call, lua_runtime, expected_globals),
             functools.partial(self._lua_redis_pcall, lua_runtime, expected_globals),
             functools.partial(_lua_redis_log, lua_runtime, expected_globals),
-            as_attrgetter(REDIS_LOG_LEVELS),
+            LUA_MODULE.as_attrgetter(REDIS_LOG_LEVELS),
         )
         expected_globals.update(lua_runtime.globals().keys())
 
@@ -201,7 +223,7 @@ class ScriptingCommandsMixin:
             if self.version < (7,):
                 raise SimpleError(msgs.SCRIPT_ERROR_MSG.format(sha1.decode(), ex))
             raise SimpleError(ex.value)
-        except LuaError as ex:
+        except LUA_MODULE.LuaError as ex:
             raise SimpleError(msgs.SCRIPT_ERROR_MSG.format(sha1.decode(), ex))
 
         _check_for_lua_globals(lua_runtime, expected_globals)
@@ -233,9 +255,7 @@ class ScriptingCommandsMixin:
 
     @command(name="SCRIPT FLUSH", fixed=(), repeat=(bytes,), flags=msgs.FLAG_NO_SCRIPT)
     def script_flush(self, *args: bytes) -> SimpleString:
-        if len(args) > 1 or (
-                len(args) == 1 and null_terminate(args[0]) not in {b"sync", b"async"}
-        ):
+        if len(args) > 1 or (len(args) == 1 and null_terminate(args[0]) not in {b"sync", b"async"}):
             raise SimpleError(msgs.BAD_SUBCOMMAND_MSG.format("SCRIPT"))
         self.script_cache = {}
         return OK
